/*ORBAS a open source CORBA implementation from Huihoo.
*
*
*Copyright 2002 Huihoo Power, Inc. All Right Reserved.
*
*This software is licensed under LGPL license.
*See terms of license at gnu.org.
*
*For more information, visit:
*
*http://www.huihoo.com
*/



package org.huihoo.orbas.orb;

import java.util.*;
import org.huihoo.orbas.poa.POA_impl;
import org.huihoo.orbas.poa.OrbasPoa;
import org.huihoo.orbas.orb.protocols.*;
import org.huihoo.orbas.orb.ior.*;
import org.huihoo.orbas.orb.cdr.*;
import org.omg.IOP.IOR;
import org.huihoo.orbas.orb.thread.*;


public class ORB extends ORBSingleton {
    private Hashtable initialReferences = new Hashtable();
    private Hashtable valueFactories = new Hashtable();
    private static final String[] services =
        {
		"RootPOA"
	};
	
    private POA_impl rootPoa;
    private ProfileRegistry profileDealers;
    private TransportsPool pool;
    private IopEngine invoker;
    private RequestQueue queue;
    private ServerWorkerRegistry handler;
    private Listener listener;
    private java.util.Hashtable poaTable;

    public ORB() { }

    public void initialize() {
        profileDealers = new ProfileRegistry();
        pool = new TransportsPool(this);
        
        org.huihoo.orbas.orb.protocols.iiop.IiopProtocolInitializer.setup(this);
        invoker = new IopEngine(this);
        
        queue = new RequestQueue(this);
        handler = new ServerWorkerRegistry(this);
        listener = new Listener(this);
        poaTable = new java.util.Hashtable();
        
        rootPoa = new POA_impl(this, null, "RootPOA", null, null);
        
    }
    
    public org.omg.CORBA.Object string_to_object(String str) {
        IorParser ior = new IorParser(this, str);
        return ior.getObject();
    }

    public String object_to_string(org.omg.CORBA.Object obj) {
        if (obj == null)
            return null;
        ObjRefDelegate delegate = (ObjRefDelegate)((org.omg.CORBA.portable.ObjectImpl) obj)._get_delegate();
        
        String ior = delegate.toString();
        return ior;
    }

    public boolean get_service_information(short service_type, org.omg.CORBA.ServiceInformationHolder service_info) {
        //we have no service supported here
        return false;
    }

    public String[] list_initial_services() {
        Vector v = new Vector();
        Enumeration e = initialReferences.keys();
        while (e.hasMoreElements()) {
            v.add(e.nextElement());
        }
        String[] initial_services = new String[services.length + v.size()];
        v.copyInto(initial_services);
        System.arraycopy(services, 0, initial_services, v.size(), services.length);
        return initial_services;
    }

    public void register_initial_reference(String object_name, org.omg.CORBA.Object object)
        throws org.omg.CORBA.ORBPackage.InvalidName {
            if (object_name == null || object_name.length() == 0 || initialReferences.containsKey(object_name)) {
                throw new org.omg.CORBA.ORBPackage.InvalidName();
            }
            else {
                initialReferences.put(object_name, object);
            }
    }

    public org.omg.CORBA.Object resolve_initial_references(String object_name) throws org.omg.CORBA.ORBPackage.InvalidName {
        if (initialReferences.containsKey(object_name)) {
            return (org.omg.CORBA.Object)initialReferences.get(object_name);
        }
        else if (object_name.equals("RootPOA")) {
            return (org.omg.CORBA.Object)getRootPOA();
        }
        else {
            //get initial references from elsewhere
            throw new org.omg.CORBA.ORBPackage.InvalidName();
        }
    }

    public void run() {
        //implementation here
        java.lang.Thread handlerThread = new java.lang.Thread(handler);
//        java.lang.Thread listenerThread = new java.lang.Thread(listener);
        handlerThread.start();
//        listenerThread.start();
        listener.run();
    }

    public void shutdown(boolean wait_for_completion) {
      
      //do all works
      
      
    }
    
    public void destroy() {
        
    }

    public org.omg.CORBA.Policy create_policy(int type, org.omg.CORBA.Any val) throws org.omg.CORBA.PolicyError {
        //implementation here
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public org.omg.CORBA.portable.OutputStream create_output_stream() {
        return new CDROutputStream(this);
    }

    protected void set_parameters(String[] args, java.util.Properties props) {
        //implementation here
        initialize();
    }

    protected void set_parameters(java.applet.Applet app, java.util.Properties props) {
        //implementation here
        initialize();
    }

    public org.omg.CORBA.Object get_value_def(String repid) {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public org.omg.CORBA.portable.ValueFactory register_value_factory(String id, org.omg.CORBA.portable.ValueFactory factory) {
        //implementation here
        return (org.omg.CORBA.portable.ValueFactory) valueFactories.put(id, factory);
    }

    public void unregister_value_factory(String id) {
        //implementation here
        valueFactories.remove(id);
    }

    public org.omg.CORBA.portable.ValueFactory lookup_value_factory(String id) {
        //implementation here
        return (org.omg.CORBA.portable.ValueFactory) valueFactories.get(id);
    }

    public void set_delegate(java.lang.Object wrapper) {
        //implementation here
        if (wrapper instanceof org.omg.PortableServer.Servant) {
            //set my own Delegate here
            
            //did we need multiple delegate instances?
            org.omg.PortableServer.Servant servant = (org.omg.PortableServer.Servant)wrapper;
            servant._set_delegate(new ServantDelegate(this));
        }
        else {
            throw new org.omg.CORBA.BAD_PARAM(0, org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    }
    
    public IorParser createIor(byte[] objectKey, String typeId) {
        //generating ior using default transport protocols
        
        org.omg.IOP.TaggedProfile[] profiles = 
            profileDealers.getProfileCreator(getDefautTransportProtocol()).createProfiles(objectKey,
                                                                                          getDefaultGiopVersion());
        
        
        IorParser parser = new IorParser(this, profiles, typeId);
        
        return parser;
    }

   
//    public org.omg.CORBA.Any create_any() {
//        //implementation here
//        return new Any(this);
//    }

    public POA_impl getRootPOA() { return rootPoa; }

    public ProfileRegistry getProfileRegistry() { return profileDealers; }
    
    public ClientWorker getDefaultClientWorker() { return new org.huihoo.orbas.orb.giop.GiopClientWorker(this); }
    
    public ServerWorker getDefaultServerWorker(Transport t) {
        return new org.huihoo.orbas.orb.giop.GiopServerWorker(this, t, 10);
    }
    
    public TransportsPool getTransportsPool() { return pool; }
    
    public int getDefaultGiopVersion() { return 10; }
    
    public int getDefautTransportProtocol() { return 0; };
    
    public IopEngine getDefaultIopEngine() { return invoker; }
    
    public RequestQueue getRequestQueue() { return queue; }
    
    public OrbasPoa locatePoa(String id) {
        //change
        return (OrbasPoa)poaTable.get(id);
    }
    
    public void registerPoa(POA_impl poa,String id) {
        poaTable.put(id, poa);
    }
    
    public void unregisterPoa(String id) {
        poaTable.remove(id);
    }
    
}
