/*ORBAS a open source CORBA implementation from Huihoo.
*
*
*Copyright 2002 Huihoo Power, Inc. All Right Reserved.
*
*This software is licensed under LGPL license.
*See terms of license at gnu.org.
*
*For more information, visit:
*
*http://www.huihoo.com
*/



package org.huihoo.orbas.orb.protocols.iiop;

import org.huihoo.orbas.orb.protocols.*;
import org.huihoo.orbas.orb.cdr.*;
import org.omg.IIOP.*;

public class IiopProfileCreator extends ProfileCreator {

    IiopChannel ch;
    
    public IiopProfileCreator(CDROutputStream out) {
        super(out);
        ch = new IiopChannel();
    }
    
    public IiopProfileCreator() {
        ch = new IiopChannel();
    }
    
    public Channel createChannel() {
        return ch;
    }
    
    public org.omg.IOP.TaggedProfile[] createProfiles(byte[] objectKey, int version) {
        
        
        org.omg.IOP.TaggedProfile[] profiles = new org.omg.IOP.TaggedProfile[1];
        org.omg.IOP.TaggedProfile profile;
        java.net.InetAddress addr = null;
        try {
            addr = java.net.InetAddress.getLocalHost();
        }
        catch(java.net.UnknownHostException e) {
            System.out.println("Error:Can not get local host address");
            e.printStackTrace();
            return null;
        }
        
        switch(version) {
            case 10:
                ProfileBody_1_0 profile10 = new ProfileBody_1_0();
                profile10.iiop_version = new org.omg.IIOP.Version();
                profile10.iiop_version.major = 1;
                profile10.iiop_version.minor = 0;
                profile10.host = addr.getHostAddress();
                profile10.port = 5000;
                profile10.object_key = objectKey;
                
                CDROutputStream out0 = new CDROutputStream(null);
                
                //byte flag cannot omitted here
                out0.write_boolean(false);
                ProfileBody_1_0Helper.write(out0, profile10);
                
                byte[] data0 = new byte[out0.getLength()];
                System.arraycopy(out0.getBuffer(), 0, data0 , 0, out0.getLength());
                profile = new org.omg.IOP.TaggedProfile(org.omg.IOP.TAG_INTERNET_IOP.value, data0);
                profiles[0] = profile;
                
                break;
                
            case 11:
                
                ProfileBody_1_1 profile11 = new ProfileBody_1_1();
                profile11.iiop_version = new org.omg.IIOP.Version();
                profile11.iiop_version.major = 1;
                profile11.iiop_version.minor = 1;
                profile11.host = addr.getHostAddress();
                profile11.port = 5000;
                profile11.object_key = objectKey;
                profile11.components = new org.omg.IOP.TaggedComponent[0];
                
                CDROutputStream out1 = new CDROutputStream(null);
                
                //byte flag cannot omitted here
                out1.write_boolean(false);
                ProfileBody_1_1Helper.write(out1, profile11);
                
                byte[] data1 = new byte[out1.getLength()];
                System.arraycopy(out1.getBuffer(), 0, data1 , 0, out1.getLength());
                profile = new org.omg.IOP.TaggedProfile(org.omg.IOP.TAG_INTERNET_IOP.value, data1);
                
                profiles[0] = profile;
                
                break;
                
            case 12:
                
                ProfileBody_1_1 profile12 = new ProfileBody_1_1();
                profile12.iiop_version = new org.omg.IIOP.Version();
                profile12.iiop_version.major = 1;
                profile12.iiop_version.minor = 2;
                profile12.host = addr.getHostAddress();
                profile12.port = 5000;
                profile12.object_key = objectKey;
                profile12.components = new org.omg.IOP.TaggedComponent[0];
                
                CDROutputStream out2 = new CDROutputStream(null);
                
                //byte flag cannot omitted here
                out2.write_boolean(false);
                ProfileBody_1_1Helper.write(out2, profile12);
                
                byte[] data2 = new byte[out2.getLength()];
                System.arraycopy(out2.getBuffer(), 0, data2 , 0, out2.getLength());
                profile = new org.omg.IOP.TaggedProfile(org.omg.IOP.TAG_INTERNET_IOP.value, data2);
                
                break;
                
            default:
                //error!
                
        }
        
        return profiles;
    }

}